from argparse import ArgumentParser
from pathlib import Path
import yaml
import userpaths
import sys
import subprocess

from app.run_config import RunConfig
from app.yaml_config import YamlConfig

sys.path.append("/home/andri/Projects/start-and-attach-vm/vbox_sdk/sdk/bindings/xpcom/python")


def get_properties_from_args(vm_name: str, port: int, **kwargs) -> RunConfig:
    return RunConfig(vm_name, port)

def get_properties_from_yaml(config: str, **kwargs) -> RunConfig:
    current_directory_config_file = Path(f"{config}.yaml")
    documents_directory_config_file = Path(userpaths.get_my_documents()) / Path("sav") / Path(f"{config}.yaml")
    full_path_config_file = Path(config)

    yaml_config: YamlConfig
    if current_directory_config_file.is_file() and current_directory_config_file.exists():
        yaml_config = get_yaml_config_from_yaml_file(current_directory_config_file)

    elif documents_directory_config_file.is_file() and documents_directory_config_file.exists():
        yaml_config = get_yaml_config_from_yaml_file(documents_directory_config_file)

    else:
        yaml_config = get_yaml_config_from_yaml_file(full_path_config_file)

    return RunConfig(yaml_config["vm_name"], yaml_config["port"])

def get_yaml_config_from_yaml_file(file: Path) -> YamlConfig:
    with open(file, "r") as f:
        return yaml.safe_load(f)
    
def start_and_attach_vm(config: RunConfig) -> None:
    subprocess.run(["VBoxManage", "startvm", f"{config.vm_name}", "--type", "headless"])
    subprocess.run(["ssh", "-X", "127.0.0.1", "-p", f"{config.localhost_port}"])

def main():
    parser = ArgumentParser("VirtualBox start and attach to VM", description="Start an VirtualBox VM and attach to it via ssh")
    subparsers = parser.add_subparsers(title="config loader")

    args_subparser = subparsers.add_parser("args", aliases=["a"], description="Pass vm properties as args")
    args_subparser.add_argument("--vm-name", "-n", action="store", required=True, type=str)
    args_subparser.add_argument("--port", "-p", action="store", required=True, type=int)
    args_subparser.set_defaults(get_properties=get_properties_from_args)

    yaml_subparser = subparsers.add_parser("yaml", aliases=["y"], description="Load vm properties from yaml")
    yaml_subparser.add_argument("config", action="store", type=str, help="Can be a yaml config in current directory, Documents/sav/, or a path to a config")
    yaml_subparser.set_defaults(get_properties=get_properties_from_yaml)

    parsed_args = parser.parse_args()
    args = vars(parsed_args)

    run_config: RunConfig = parsed_args.get_properties(**args)
    start_and_attach_vm(run_config)


if __name__ == "__main__":
    main()
