from setuptools import setup, find_packages
from setuptools.command.install import install
from urllib import request
from pathlib import Path
import re
from functools import reduce
from zipfile import ZipFile
import subprocess
from typing import Dict, Any
import os

# url must be a URL, but package is not available at this time
def get_links_from_url(url: Any, pattern: str, name_pattern: str | None = None, name_position: int = 1) -> Dict[str, Any]:
    page_request = request.urlopen(str(url))
    page_content = page_request.read().decode('utf-8')
    link_pattern = r"<a href=\"(.*?)\">(.*?)</a>"
    matches = re.findall(link_pattern, page_content)

    link_info = {}
    for match in matches:
        matched, link = match
        formatted_match = matched.strip().replace("/", "")

        search_result = re.match(pattern, formatted_match)
        if search_result is not None:
            name = search_result.group(0)
            if name_pattern is not None:
                name = re.search(name_pattern, name).group(name_position)

            link_info[name] = link

    return link_info

def setup_virtualbox_sdk():
    from urlpath import URL
    import semver

    sdk_zip_path = Path("sdk.zip")
    sdk_path = Path("vbox_sdk")

    if not sdk_path.exists():
        if not sdk_zip_path.exists():
            virtualbox_download_url = URL("https://download.virtualbox.org/virtualbox/")
            version_pattern = r"(\d+\.\d+\.\d+)(?!\S)"
            link_info = get_links_from_url(virtualbox_download_url, version_pattern)
            
            highest_version = reduce(semver.max_ver, link_info.keys())
            highest_version_link = virtualbox_download_url / URL(link_info[highest_version])
            sdk_pattern = r"VirtualBoxSDK-" + re.escape(highest_version) + "-(\d+)\.zip"
            sdk_version_pattern = r"-(\d+)\.zip"
            sdk_link_info = get_links_from_url(highest_version_link, sdk_pattern, name_pattern=sdk_version_pattern)
            highest_sdk_version = reduce(semver.max_ver, sdk_link_info.keys())

            sdk_download_link = highest_version_link / URL(sdk_link_info[highest_sdk_version])
            print(sdk_download_link)

            request.urlretrieve(str(sdk_download_link), str(sdk_zip_path))

        with ZipFile(str(sdk_zip_path), "r") as f:
            f.extractall(sdk_path)

        os.remove(sdk_zip_path)
        # set VBOX_SDK_PATH env variable
        # https://github.com/sethmlarson/virtualbox-python/issues/104#issuecomment-358443645

    subprocess.run(["python", "vboxapisetup.py", "install"], cwd="vbox_sdk/sdk/installer/")

class InstallCommand(install):
    def run(self):
        # setup_virtualbox_sdk()

        install.run(self)

requirements = []
with open("requirements.txt") as f:
    requirements = f.readlines()

setup(name='sav',
    version='1.0',
    packages=find_packages(),
    install_requires=requirements,
    entry_points={
        "console_scripts": [
            "sav = app.main:main"
        ],
    },
    setup_requires=[
        "setuptools",
        "semver",
        "urlpath",
    ],
    cmdclass={
        "install": InstallCommand,
    }
)

# vbox_sdk_path = Path(".") / Path("vbox_sdk")
# if not vbox_sdk_path.exists():
#     setup_virtualbox_sdk()
